--
-- Created by IntelliJ IDEA.
-- User: admin
-- Date: 7/07/2014
-- Time: 3:34 PM
-- To change this template use File | Settings | File Templates.
--

local Ace = LibStub("AceAddon-3.0")
local Comergy2 = Ace:GetAddon("Comergy2")

--noinspection UnusedDef
Comergy2.prototype_secondary = {

    POWER_TYPE = SPELL_POWER_CHI,
    POWER_TYPE_TEXT = "CHI",
    flashTimes = -1,

    opts = {
        priority = 40,
        name = "Chi",
        bars = 5,
    },

    defaults = {
        profile = {
            color0 = { 0.5, 0.5, 0.5 },
            color1 = { 1.0, 0.0, 0.0 },
            color2 = { 1.0, 0.5, 0.0 },
            color3 = { 1.0, 1.0, 0.0 },
            color4 = { 0.0, 1.0, 0.0 },
            color5 = { 0.0, 0.5, 1.0 },
            sound1 = 'None',
            sound2 = 'None',
            sound3 = 'None',
            sound4 = 'None',
            sound5 = 'Piano5',
            height = 10,
            bgAlpha = 0.2,
            fancy = 'none',
            uni = false,
            text = true,
            flash = true,
        }
    },

    InitializeModule = function(self)

        self.db = Comergy2.db:RegisterNamespace("Module_"..self.opts.name, self.defaults)
--        self.pwr = self:GetPower()
        self.max = self:GetMaxPower()
        if self.max == 0 then
            self.max = self.opts.bars
        end

        self:CreateFrame()
        self:ResizeFrame()
        self.frame.priority = self.opts.priority

        self:CreateBars(self.opts.bars)
        self:ResizeBars()

        self:UpdateTextures()
        self:ColorBars()
        self:CreateAnimations()
    end,

    GetPower = function(self)
        return UnitPower("player", self.POWER_TYPE)
    end,

    GetMaxPower = function(self)
        return UnitPowerMax("player", self.POWER_TYPE)
    end,

    UpdateTextures = function(self)
        local tex = Comergy2.SM3:Fetch("statusbar", Comergy2.db.profile.texture)
        for i = 1, #self.bars do
            self.bars[i]:SetStatusBarTexture(tex)
            self.bars[i].bg:SetTexture(tex)
        end
    end,

    OnEnable = function(self)
        Comergy2:Print(self:GetName(), "enable")
    --        self:RegisterEvent("UNIT_POWER")
        self:RegisterEvent("UNIT_MAXPOWER")
    end,

    OnDisable = function(self)
    --        self:UnregisterEvent("UNIT_POWER")
        self:UnregisterEvent("UNIT_MAXPOWER")
    end,

    --    UNIT_POWER = function(self, eventName, unit, powerType)
    --    end,
    UNIT_MAXPOWER = function(self, eventName, unit, powerType)
        self.max = self:GetMaxPower()
        self:ResizeBars()
    end,

    ColorBars = function(self)
        local clr
        for i = 1, #self.bars do
            if self.db.profile.uni then
                --noinspection UnusedDef
                clr = self.db.profile["color" .. self.pwr]
            else
                clr = self.db.profile["color" .. i]
            end
            self.bars[i]:SetStatusBarColor(clr[1], clr[2], clr[3])
            self.bars[i].bg:SetVertexColor(clr[1], clr[2], clr[3], self.db.profile.bgAlpha)
        end
    end,

--    Name Conflicts?
    CreateAnimations = function(self)
        local group = self.frame:CreateAnimationGroup(self:GetName().."_FlashAnimation")
        local alpha = group:CreateAnimation("Alpha")
        alpha:SetChange(-1)
        alpha:SetDuration(0.2)
        group:SetLooping("BOUNCE")
        group:SetScript("OnLoop", function(...)
            self:OnLoop(...)
        end)
        self.flash = group
    end,

    OnLoop = function(self, widget, loopState)
        if loopState == "REVERSE" then
            self.flashTimes = self.flashTimes - 1
            if self.flashTimes < 1 then -- allow for negative flashTimes
                self.flash:Finish()
            end
        end
    end,

    OnUpdate = function(self)
--        Comergy2:Print('update')
        self:UpdatePower()
        self:UpdateFlash()
    end,

    UpdatePower = function(self)
        local pwr = self:GetPower()
        if pwr ~= self.pwr then
            self.pwr = pwr
            for i = 1, self.opts.bars do
                self.bars[i]:SetValue(pwr)
            end
            if self.db.profile.uni then
                self:ColorBars()
            end
            local sound = Comergy2.SM3:Fetch("sound", self.db.profile['sound'..pwr])
            PlaySoundFile(sound)
        end
    end,

    UpdateFlash = function(self)
        if self.db.profile.flash and self.pwr == self.max then
            if self.flash and not self.flash:IsPlaying() and self.flashTimes == -1 then
                self.flashTimes = 2
                self.flash:Play()
            end
        else
            --            Do we need this?
            self.flashTimes = -1
        end
    end,

    CreateFrame = function(self)
        self.frame = CreateFrame("Frame", self:GetName(), Comergy2.frame)
        self.frame:Show()

        self.frame:SetPoint("LEFT")
        self.frame:SetPoint("RIGHT")

        self.frame:SetScript("OnUpdate",
            function(...)
                self:OnUpdate(...)
            end)

        table.insert(Comergy2.frames, self.frame)
    end,

    ResizeFrame = function(self)
        self.frame.h = self.db.profile.height + Comergy2.db.profile.spacing
        self.frame:SetHeight(self.frame.h)
        Comergy2:Resize()
    end,

    CreateBars = function(self, num)
        self.cells = {}
        self.bars = {}
        for i = 1, num do
            local cell = CreateFrame("Frame", self:GetName().."Cell"..i, self.frame)
            cell.bg = cell:CreateTexture(nil, "BACKGROUND")
            cell.bg:SetTexture(1, 1, 1, 0)

            local thickness = Comergy2.db.profile.spacing / 2 - 1
            cell.bg:SetPoint("TOPLEFT", thickness, -thickness)
            cell.bg:SetPoint("BOTTOMRIGHT", -thickness, thickness)

            self.cells[i] = cell

            local bar = CreateFrame("StatusBar", self:GetName() .. "Bar" .. i, cell)
            bar:SetMinMaxValues(i - 1, i)


            bar:SetPoint("TOPLEFT", Comergy2.db.profile.spacing/2, -Comergy2.db.profile.spacing/2)
            bar:SetPoint("BOTTOMRIGHT", -Comergy2.db.profile.spacing/2, Comergy2.db.profile.spacing/2)

            bar.bg = bar:CreateTexture(nil, "BACKGROUND", 3)
            bar.bg:SetAllPoints()

            self.bars[i] = bar
        end
    end,

    ResizeBars = function(self)
        local cell, bar
        for i = 1, #self.bars do
            cell = self.cells[i]
            bar = self.bars[i]
            cell:SetWidth((self.frame:GetWidth() - Comergy2.db.profile.spacing) / self.max)
            bar:SetOrientation("HORIZONTAL")
            --        bar:SetRotatesTexture(false) -- idk this broked
            cell:ClearAllPoints()
            cell:SetPoint("TOP")
            cell:SetPoint("BOTTOM")
            if i == 1 then
                cell:SetPoint("LEFT", Comergy2.db.profile.spacing/2, 0)
            else
                cell:SetPoint("LEFT", self.cells[i - 1], "RIGHT")
            end
            if i == self.max then
                cell:SetPoint("RIGHT", Comergy2.db.profile.spacing/-2, 0)
            end
            if i > self.max then
                cell:Hide()
            else
                cell:Show()
            end
        end
    end,

    RefreshConfig = function(self)
        self:ResizeFrame()
        self:ResizeBars()

        self:UpdateTextures()
        self:ColorBars()
        self:CreateAnimations()
    end,

    IsStable = function(self) return self.pwr == 0 end,

    Get = function(self, info)
        return self.db.profile[info[#info]]
    end,

    Set = function(self, info, value)
        self.db.profile[info[#info]] = value
        self:RefreshConfig()
    end,

    SetPosition = function(self, info, value)
        self.db.profile["position"][info[#info]] = value
        self:RefreshConfig()
    end,

    GetPosition = function(self, info)
        return self.db.profile["position"][info[#info]]
    end,

    GetColor = function(self, info)
        local color = self.db.profile[info[#info]] or {1, 1, 1, 1}
        return color[1], color[2], color[3], color[4]
    end,

    SetColor = function(self, info, r, g, b, a)
        self.db.profile[info[#info]] = {r, g, b, a}
        self:RefreshConfig()
    end,

    GetColorText = function(self, info)
        local idx = info[#info]:sub(1, -5)
        local color = self.db.profile[idx] or {1, 1, 1, 1}
        return format('%02X%02X%02X', color[1]*255, color[2]*255, color[3]*255)
    end,

    SetColorText = function(self, info, value)
        local a = 1
        local r = tonumber(value:sub(1,2), 16) / 255
        local g = tonumber(value:sub(3,4), 16) / 255
        local b = tonumber(value:sub(5,6), 16) / 255
        self.db.profile[info[#info]:sub(1, -5)] = {r, g, b, a}
        self:RefreshConfig()
    end,

    --noinspection UnusedDef
    GetEnable = function(self, info)
        return self:IsEnabled()
    end,

    --noinspection UnusedDef
    SetEnable = function(self, info, value)
        if value then
            self:Enable()
        else
            self:Disable()
        end
    end,

    SetSM3 = function(self, info, value)
        self.db.profile[info[#info]] = value
    end,

    GetOptions = function(self)
        local opts = {
            type = 'group',
            order = 10,
            args = {
--                color0Text = {
--                    type = 'input',
--                    name = 'Color 0',
--                    width = 'normal',
--                    set = 'SetColorText',
--                    get = 'GetColorText',
--                    order = 1,
--                },
                color0 = {
                    type = 'color',
                    name = 'Color 0',
                    desc = 'Color of bar',
                    set = 'SetColor',
                    get = 'GetColor',
                    order = 2,
                    width = 'full',
                },
                uni = {
                    type = 'toggle',
                    name = 'Unified Bar Color',
                    desc = 'Color all bars the same',
                    order = 301,
                },
                flash = {
                    type = 'toggle',
                    name = 'Flash when full',
                    desc = 'Flash bars when full',
                    order = 302,
                },
                height = {
                    type = 'range',
                    name = 'Bar thickness',
                    desc = 'Height of the bars',
                    min = 1,
                    max = 100,
                    softMax = 30,
                    step = 1,
                    width = 'normal',
                    order = 201,
                },
                bgAlpha = {
                    type = 'range',
                    name = 'BG Alpha',
                    desc = 'brightness of depleted bars',
                    min = 0,
                    max = 1,
                    step = .05,
                    width = 'normal',
                    order = 202,
                },
            },
        }
        local order = 10
        for i = 1, self.opts.bars do
--            opts.args['color'..i..'Text'] = {
--                type = 'input',
--                name = 'Color '..i,
--                width = 'normal',
--                get = 'GetColorText',
--                set = 'SetColorText',
--                order = order + 1,
--            }
            opts.args['color'..i] = {
                type = 'color',
                name = 'Color '..i,
                desc = 'Color of bar',
                set = 'SetColor',
                get = 'GetColor',
                order = order + 2,
                width = 'normal',
            }
            opts.args['sound'..i] = {
                type = 'select',
                dialogControl = 'LSM30_Sound', --Select your widget here
                name = 'Sound '..i,
                desc = 'Sound when full',
                values = AceGUIWidgetLSMlists.sound, -- pull in your font list from LSM
                order = order + 3,
            }
            order = order + 10
        end

        return opts
    end
}
